@@ -592,9 +592,17 @@ instantiate_pattern(FcPattern *pattern, double req_pt_size, double req_px_size,
         return false;
     }
 
-    if ((ft_err = FT_Set_Pixel_Sizes(ft_face, 0, round(pixel_size))) != FT_Err_Ok) {
-        LOG_ERR("%s: failed to set character size: %s",
-                face_file, ft_error_string(ft_err));
+    /*
+     * Using FT_Set_Char_Size() instead of FT_Set_Pixel_Sizes() allows
+     * us to use "fractional" pixel values. This _does_ make a
+     * difference.
+     *
+     * Since FT_Set_Char_Size() expects a size in *points*, not
+     * pixels, we use a DPI of 72. This means a pixel have the same
+     * size as a point.
+     */
+    if ((ft_err = FT_Set_Char_Size(ft_face, pixel_size * 64., 0, 72, 72))) {
+        LOG_ERR("%s: failed to set character size: %s", face_file, ft_error_string(ft_err));
         goto err_done_face;
     }
 
@@ -763,6 +771,8 @@ instantiate_pattern(FcPattern *pattern, double req_pt_size, double req_px_size,
     char features[256] = {0};
 
 #if defined(FCFT_HAVE_HARFBUZZ)
+    font->hb_feats_count = 0;
+
     font->hb_font = hb_ft_font_create_referenced(ft_face);
     if (font->hb_font == NULL) {
         LOG_ERR("%s: failed to instantiate harfbuzz font", face_file);
@@ -775,32 +785,32 @@ instantiate_pattern(FcPattern *pattern, double req_pt_size, double req_px_size,
         goto err_hb_font_destroy;
     }
 
-    for (font->hb_feats_count = 0; font->hb_feats_count < ALEN(font->hb_feats); ) {
+    for (size_t feat_no = 0; feat_no < ALEN(font->hb_feats); feat_no++) {
         FcChar8 *fc_feat;
         if (FcPatternGetString(
-                pattern, FC_FONT_FEATURES,
-                font->hb_feats_count, &fc_feat) != FcResultMatch)
+            pattern, FC_FONT_FEATURES, feat_no, &fc_feat) != FcResultMatch)
         {
             break;
         }
 
         hb_feature_t *feat = &font->hb_feats[font->hb_feats_count];
 
-        if (hb_feature_from_string((const char *)fc_feat, -1, feat)) {
-            const char tag[4] = {
-                (feat->tag >> 24) & 0xff,
-                (feat->tag >> 16) & 0xff,
-                (feat->tag >> 8) & 0xff,
-                (feat->tag >> 0) & 0xff,
-            };
+        if (!hb_feature_from_string((const char *)fc_feat, -1, feat))
+            continue;
 
-            size_t ofs = font->hb_feats_count * 6;
-            snprintf(&features[ofs], sizeof(features) - ofs,
-                     " %c%.4s", feat->value ? '+' : '-', tag);
+        const char tag[4] = {
+            (feat->tag >> 24) & 0xff,
+            (feat->tag >> 16) & 0xff,
+            (feat->tag >> 8) & 0xff,
+            (feat->tag >> 0) & 0xff,
+        };
 
-            LOG_DBG("feature: %.4s=%s", tag, feat->value ? "on" : "off");
-            font->hb_feats_count++;
-        }
+        size_t ofs = font->hb_feats_count * 6;
+        snprintf(&features[ofs], sizeof(features) - ofs,
+                 " %c%.4s", feat->value ? '+' : '-', tag);
+
+        LOG_DBG("feature: %.4s=%s", tag, feat->value ? "on" : "off");
+        font->hb_feats_count++;
     }
 #endif
 
@@ -835,8 +845,8 @@ instantiate_pattern(FcPattern *pattern, double req_pt_size, double req_px_size,
             font->metrics.height, font->metrics.ascent, font->metrics.descent,
             font->metrics.max_advance.x, features);
 #else
-    LOG_INFO("%s: size=%.2fpt/%dpx, dpi=%.2f%s",
-             font->path, size, (int)round(pixel_size), dpi, features);
+    LOG_INFO("%s: size=%.2fpt/%.2fpx, dpi=%.2f%s",
+             font->path, size, pixel_size, dpi, features);
 #endif
 
     return true;
